;MXFLASH0.ASM
;Firmware-based Flash utility for AM29F010 on Adapt-812 MX1 Memory Expansion
;Instructions:
;============
;Install this code (MXFLASH0.S19) in 68HC812A4 on-chip EEPROM
;Then reset with MODA & MODB jumpered for single-chip mode (ie. both low)
;This code switches on external bus in page mode and accesses external Flash
;connected to CSP0*.  To use pages, user must embed page number in s-record
;prior to the s-record segment to be loaded into a given page
;This can be done with assembler directives.  For example, to place a group
;of subroutines in PPAGE3 program space, preface the group of routines
;as follows:
;	org	PPAGE
;	fcb	$03
;	org	$8000
;put subroutines here...
;
;The user should always preface the vector block with:
;
;	org	PPAGE
;	fcb	$07
;	org	$bf80	;start of vector table
;put vectors here...
;
;When Flash has been programmed, jumper MODA & MODB for Normal Expanded Narrow
;mode (ie. MODA high) then reset.  The internal EEPROM will move to $1000, and
;the user code in Flash will now execute.
;==============================================================================
;
;REGS.INC IS EXTERNAL FILE CONTAINING ALL REGISTERS
;
*******************************************************************************
*REVISION HISTORY:
*
*DATE                   REV. NO.        DESCRIPTION
*
*Setp 11, 1998          1.00            Initial release
*
*******************************************************************************
;
; ----------------------------
; ADAPT-812MX1 Flash Utilities
; ----------------------------

#include REGS.INC

*  Operational Parameters

_100MS          equ     25
_250MS          equ     61
_500MS          equ     125
_1SECOND        equ     250
_2SECONDS       equ     500
_3SECONDS       equ     750
_5SECONDS       equ     1250
_10SECONDS      equ     2500
_25SECONDS      equ     6250
_60SECONDS      equ     15000
_2MIN           equ     30000

RAM     equ     $0800           ;68HC812A4 internal RAM
STACK   equ     $0c00           ;Stack at top of internal ram
rbase   equ     $0000           ;68HC812A4 register block
EEPROM  equ     $f000           ;68HC812A4 internal EEPROM in SCM
SRAM	equ	$7000		;external SRAM memory window
FLASH	equ	$8000		;external Flash memory window
LAST_PAGE	equ	$7	;pages 0 thru 7 (8 x 16K pages in 128K Flash)

* Operational Constants

true            equ     $ff
false           equ     $00
CR              equ     $D
LF              equ     $A
SPACE           equ     $20

* Masks

;SCI Variables
scimask         equ     %00101100 ;RIE - SCI Interrupt enable
                                  ;RE - Receiver Enable
RDRFflag        equ     %00100000 ;RDRF - Receive Data Register Full flag
TDREflag        equ     %10000000 ;TDRE - Transmit Data Register Empty flag

;Baud rate definitions
;MCLK=8MHz
BAUD110         equ     4545    ;(baud) 110 baud with 16 Mhz crystal
BAUD300         equ     1667    ;(baud) 300 baud with 16 Mhz crystal
BAUD600         equ     833     ;(baud) 600 baud with 16 Mhz crystal
BAUD1200        equ     417     ;(baud) 1200 baud with 16 Mhz crystal
BAUD2400        equ     208     ;(baud) 2400 baud with 16 Mhz crystal
BAUD4800        equ     104     ;(baud) 4800 baud with 16 Mhz crystal
BAUD9600        equ     52      ;(baud) 9600 baud with 16 Mhz crystal
BAUD14400       equ     35      ;(baud) 14400 baud with 16 Mhz crystal
BAUD19200       equ     26      ;(baud) 19200 baud with 16 Mhz crystal
BAUD38400       equ     13      ;(baud) 38400 baud with 16 Mhz crystal

        org     RAM

;-------------
;RTC Variables
;-------------
spi_read                ds      1
spi_write               ds      1

rtc_reg_address         ds      1
read_write_flag         ds      1               ;if bit 0 is 0 then write else read
ack_flag                ds      1               ;acknowledged flag

;time variables
time_temp
time_seconds_hundred    ds      1
time_seconds            ds      1
time_minutes            ds      1
time_hours              ds      1
time_days               ds      1
time_months             ds      1
time_years              ds      1

time_flag               ds      1
date_flag               ds      1

* System Variables

BYTE		rmb	1	;used for data polling

TEMP1		rmb	1
TEMP2		rmb	1

CURPAGE		rmb	1	;current page counter

H               rmb     1       ;used in binary to decimal conversion
TO              rmb     1


* Boolean Variables

ERROR		rmb	1	;Flash operation status
WRITE_MODE	rmb	1	;write or verify

***************************** Program *******************************

        org     EEPROM

Init:	                          ;This is where the RESET vector points to
;Initialize Stack
        lds     #STACK                  ;initialize stack pointer

;Set up external memory control registers
	movb	#$04,PEAR	;enable read/write signal
	movb	#$b0,MODE	;normal exp. narrow
	movb	#$c0,WINDEF	;enable data & program windows
        movb    #$07,MXAR       ;enable A16, A17, A18
	movb	#$30,CSCTL0	;enable CSD & CSP0
	movb	#$10,CSCTL1	;CSD covers lower half of memory map
	movb	#$f1,INITEE	;make sure EEPROM is at top

;Initialize COP
	clr	COPCTL			;turn COP off

;Initialize Serial Communication Interface
        movb    #$0c,SC0CR2		;enable SCI 0 rx & tx
        movb    #0,SC0CR1		;  for polled operation
        movw    #BAUD9600,SC0BDH	;Set baud rate

	movb	#$ff,DDRT		;make port t outputs

        jsr     RTC_INIT

main
	jsr	ShowMenu	;display menu in terminal window
	jsr     ProcessCommand  ;this is the program loop
        bra     main


***********************************************************************
*Procedure Definitions:
***********************************************************************

ShowMenu:
        jsr     PutDblLine
        ldx     #M_UNDLN
        jsr     SendString
        jsr     PutNewLine
        jsr     PutNewLine
        ldx     #M_HEADER1
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_UNDLN
        jsr     SendString
        jsr     PutDblLine

        ldx     #M_ERASE
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_PROGRAM
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_TIME
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_DATE
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_VERIFY
        jsr     SendString
        jsr     PutNewLine
        ldx     #M_PROMPT
        jsr     SendString
        rts


ProcessCommand:
	jsr	GetChar
	jsr	SendByte	;echo the character
	oraa	#$20		;convert to lower case, if required

        cmpa    #'e'
        beq     PCE

        cmpa    #'p'
        beq     PCP

        cmpa    #'t'
        beq     PCT

        cmpa    #'d'
        beq     PCD

        cmpa    #'v'
        beq     PCV

	bra	PCX		;  and exit

PCE:
	jsr	PutNewLine
	ldx	#M_ERASING
	jsr	SendString
	jsr	PutNewLine
        jsr	EraseFlash	;erase Flash
        bra     PCX
PCP:
	jsr     PutNewLine
	ldx	#M_PROMPT1
	jsr	SendString
	jsr     PutNewLine
	movb	#true,WRITE_MODE	;select WRITE mode
        jsr	ProcessSRecord
	tst	ERROR
	bne	PCP_Fail
	ldx	#M_PROG_OK
	bra	PCP_SS
PCP_Fail:
	ldx	#M_PROG_FAIL
PCP_SS:
	jsr	SendString
        bra     PCX

PCT:
	jsr	PutNewLine
        jsr     GET_TIME
	bra	PCX

PCD:
	jsr	PutNewLine
        jsr     GET_DATE
	bra	PCX

PCV:
	jsr     PutNewLine
	clr	WRITE_MODE	;select verify mode
	ldx	#M_SEND_S19
	jsr	SendString
	jsr     PutNewLine
        jsr	ProcessSRecord
	jsr     PutNewLine
	tst	ERROR
	bne	PCV_Fail
	ldx	#M_VERIFY_OK
	bra	PCV_SS
PCV_Fail:
	ldx	#M_VERIFY_FAIL
PCV_SS:
	jsr	SendString
        bra     PCX

PCX:
        rts


EraseFlash:
	movb	#$ff,PORTT	;LED on
	clr	ERROR
	movb	#$1,PPAGE
	movb	#$aa,$9555
	clr	PPAGE
	movb	#$55,$aaaa
	movb	#$1,PPAGE
	movb	#$80,$9555
	movb	#$aa,$9555
	clr	PPAGE
	movb	#$55,$aaaa
	movb	#$1,PPAGE
	movb	#$10,$9555
	clr	PPAGE
	ldy	#FLASH
	movb	#$ff,FLASH
EF1:
	ldab	0,y		;msb of Flash byte is low until erased
	bmi	EF_OK
	ldab	#$20		;DQ5=1 if write times out
	bitb	0,y
	beq	EF1
	ldab	0,y		;check once more
	bmi	EF_OK
	ldx	#M_ERASE_FAIL	;  send FAIL message
	bra	EF_SS
EF_OK:
	ldx	#M_ERASE_OK	;  send OK message
EF_SS:
	clr	PORTT		;LED off
	jsr	SendString
        jsr     PutNewLine
	rts


TestFlash:
PCT1:
	ldy	#FLASH
PCT2:				;write command sequence
	ldaa	#$a5
	clr	ERROR
	jsr	WriteFlash
	tst	ERROR
	bne	PCT_Fail
PCT_OK:
	iny
	cpy	#$c000		;until end of page
	bne	PCT2
Done:
	ldx	#M_TEST_OK	;  send OK message
	bra	PCT_SS
PCT_Fail:
	ldx	#M_TEST_FAIL	;  send FAIL message
PCT_SS:
	jsr	SendString
	jsr     PutNewLine
	rts


ProcessSRecord:
	clr	ERROR		;no errors yet
M0:
	jsr	GetChar		;look for S record
	cmpa	#'S
	bne	M0
	jsr	GetChar
	cmpa	#'1		;if not S1 record then
	beq	Load1

	cmpa	#'9		;if S9 record then
	bne	M0
	jsr	GetByte		;get byte count
	tab
	subb	#2
	jsr	GetAddress	;get execution address
Load9:
	jsr	GetByte		;get checksum byte (and ignore it)
	decb
	bne	Load9
	bra	PSX
Load1:
	jsr	GetByte		;S1 record, so get byte count
	tab
	subb	#3		;adjust count for address and checksum bytes
	jsr	GetAddress	;get address
	dey			;adjust address for L1B loop
	bra	L1B
L1B:				;repeat
	jsr	GetByte		;  get a byte (in A)
	iny			;  increment target address
	tst	WRITE_MODE	;  if WRITE_MODE then
	beq	VF
	cpy	#FLASH		;    if address below Flash then
	bhs	L1C
	staa	0,y		;      do normal write
	bra	VF
L1C:
        staa    BYTE
	jsr	WriteFlash	;     else do Flash write
	ldaa	BYTE
VF:
	cmpa	0,y		;  if byte verified then
	bne	VerfErr
	decb			;  decrement # of bytes
	bne	L1B		;until all bytes in record programmed
	clr	PORTT		;turn off LED
	bra	M0		;get next S-record
VerfErr:
	inc	ERROR		;set error flag
PSX:
	rts

WriteFlash:
	movb	PPAGE,CURPAGE	;preserve PPAGE value
	clr	PPAGE		;Flash needs to see address of $5555,
				;which is in second 16K page of Flash
	inc	PPAGE		;so PPAGE=1 for $4000 offset into flash
	movb	#$aa,$9555	;need offset of $8000 to generate CSP0*
	clr	PPAGE		;for Flash CE* then add $1555 ($5555-$4000)
	movb	#$55,$aaaa	;Flash needs address $2aaa (in first page)
				;so just add $8000 to generate CSP0* for CE*
	inc	PPAGE		;same logic as before
	movb	#$a0,$9555
	movb	CURPAGE,PPAGE	;restore PPAGE value

        staa    0,y             ;write Flash
 
DataPoll:		;repeat
        ldaa    BYTE    ;get value being programmed
        cmpa    0,y
        beq     DPX

CheckDQ5:
        brset   0,y,0010000,DP_Fail
        bra     DataPoll
 
DP_Fail:
        nop
        nop
        nop

	ldaa	BYTE	;check one last time before giving up
        cmpa    0,y
        beq     DPX

;        eora    0,y
;        bpl     DPX

	movb	#$ff,PORTT	;LED on
	inc	ERROR
DPX:
	rts

;-----------------------------------------------------------------------------
HexByte2Dec

			;Supply hex value to be converted in B register
        clra		;numerator in D, denominator in X
        ldx     #100    ;remainder in D, quotient in X
        idiv		;B is lo-order byte
        xgdx
        stab    H       ;8 bit quotient in B
        xgdx
        ldx     #10     ;H=0X  TO=packed BCD
        idiv
        stab    TO
        xgdx
        aslb
        aslb
        aslb
        aslb
        orab    TO
        stab    TO
        rts


GetChar:

*  Fetches one character from SCI0, 
*  and returns it in register A.  

GC1:
	ldaa	SC0SR1		;get sci flags
	anda	#RDRFflag	;mask off irrelevant bits
	beq	GC1		;loop until character found
	ldaa	SC0DRL		;get character
	rts


GetByte:

*  Converts 2 hex characters to a single byte value, 
*  and returns it in register A.

GB1:
	jsr	GetChar
	jsr	Hex2Bin
	lsla
	lsla
	lsla
	lsla
	staa	TEMP1
	jsr	GetChar
	jsr	Hex2Bin
	oraa	TEMP1
	rts


Hex2Bin:

*  Converts an ASCII hex character in register A
*  to a binary nibble and returns it in register A.

	cmpa	#'9
	bls	HEX
	adda	#9
HEX:	anda	#$f
	rts

GetAddress:

* Extracts load address and returns it in register Y.

	pshb			;preserve byte count
	jsr	GetByte
	staa	TEMP2
	jsr	GetByte
	tab
	ldaa	TEMP2
	xgdy
	pulb
	rts	


SendDecByte

* value is in B

        clra
        jsr     HexByte2Dec
        ldaa    H
        adda    #'0'
        jsr     SendByte
        ldaa    TO
        jsr     SendASCIIHex
        rts

SendASCIIHex
*       value in A

        psha
        anda    #$F0
        lsra
        lsra
        lsra
        lsra
        cmpa    #$A
        blt     SAH1
        suba    #$A
        adda    #'A'
        bra     SAH2
SAH1
        adda    #'0'
SAH2
        jsr     SendByte
        pula
        psha
        anda    #$0F
        cmpa    #$A
        blt     SAH3
        suba    #$A
        adda    #'A'
        bra     SAH4
SAH3
        adda    #'0'
SAH4
        jsr     SendByte
        pula
        rts


SendByte:

*  Transmits a byte, contained in register A, to the serial device via the SCI.

SB1
        ldab    SC0SR1  ;wait for transmit data register empty (TDRE)
        bpl     SB1
        staa    SC0DRL  ;send byte
        rts

PutNewLine
        ldaa    #CR
        jsr     SendByte
        ldaa    #LF
        jsr     SendByte
        rts

PutDblLine
        ldaa    #CR
        jsr     SendByte
        ldaa    #LF
        jsr     SendByte
        ldaa    #LF
        jsr     SendByte
        rts

SendString:
*       x contains starting address of string
RV1
        ldaa    0,X             ;$FF denotes end of string
        cmpa    #$ff
        beq     RVX
        jsr     SendByte
        inx
        bra     RV1
RVX
        rts

*******************************************************************************
* Messages

*******************************************************************************
* Messages
M_ERASE         fcc     'E => Erase entire Flash chip'
                fcb     $ff
M_PROGRAM       fcc     'P => Program s-record into Flash'
                fcb     $ff
M_TIME          fcc     'T => Set Time (hh:mm:ss) [24-hour format]'
                fcb     $ff
M_DATE          fcc     'D => Set Date (yy/mm/dd)'
                fcb     $ff
M_VERIFY        fcc     'V => Verify s-record matches Flash'
                fcb     $ff
M_PROMPT        fcc     '?'
                fcb     $ff
M_PROMPT1	fcc     'Send s-record to be programmed'
                fcb     $ff
M_HEADER1       fcc     '   ADAPT-812 MX1 FLASH UTILITY VER. 0'
                fcb     $ff
M_UNDLN         fcc     '_______________________________________________'
                fcb     $ff
M_ERASE_OK	fcc     'Flash erased OK'
                fcb     $ff
M_ERASE_FAIL    fcc     'Could not erase Flash'
                fcb     $ff
M_ERASING	fcc     'erasing Flash now...'
                fcb     $ff
M_TESTING	fcc     'testing Flash now...'
                fcb     $ff
M_VERIFY_OK     fcc     's-record verified OK'
                fcb     $ff
M_VERIFY_FAIL   fcc     's-record verification failed'
                fcb     $ff
M_PROG_OK       fcc     'Flash programmed OK'
                fcb     $ff
M_PROG_FAIL     fcc     'Flash programming failed'
                fcb     $ff
M_TEST_OK       fcc     'Flash test successful'
                fcb     $ff
M_TEST_FAIL     fcc     'Flash test failed'
                fcb     $ff
M_SEND_S19	fcc	'Send s-record to be verified'
		fcb	$ff
M_CURRENT_TIME  fcc     'Current Time: '
		fcb	$ff
M_NEW_TIME      fcc     'New Time: '
		fcb	$ff
M_CURRENT_DATE  fcc     'Current Date: '
		fcb	$ff
M_NEW_DATE      fcc     'New Date: '
		fcb	$ff
;
;-----------------------------------------------------------------------------
;These sub-routines are external to this main routine and are link during
; compiling/assembling stage.

#include RTC.ASM

;MC68HC812A4 VECTOR INTERRUPTS

	org	$FFCE
VECTORS:
        FDB     Init           ;KEY WAKEUP H
        FDB     Init           ;KEY WAKEUP J
        FDB     Init           ;ANALOG TO DIGITAL
        FDB     Init           ;SERIAL COMMUNICATION 1
        FDB     Init	       ;SERIAL COMMUNICATION 0
        FDB     Init           ;SPI SERIAL TRANSFER COMPLETE
        FDB     Init           ;PULSE ACCUMULATOR INPUT EDGE
        FDB     Init           ;PULSE ACCUMULATOR OVERFLOW
        FDB     Init           ;TIMER OVERFLOW
        FDB     Init           ;TIMER CHANNEL 7
        FDB     Init           ;TIMER CHANNEL 6
        FDB     Init           ;TIMER CHANNEL 5
        FDB     Init           ;TIMER CHANNEL 4
        FDB     Init           ;TIMER CHANNEL 3
        FDB     Init           ;TIMER CHANNEL 2
        FDB     Init           ;TIMER CHANNEL 1
        FDB     Init           ;TIMER CHANNEL 0
        FDB     Init	       ;REAL TIME INTERRUPT
        FDB     Init           ;IRQ OR KEY WAKE UP D
        FDB     Init           ;XIRQ
        FDB     Init           ;SWI
        FDB     Init           ;RESERVED
        FDB     Init           ;COP FAILURE RESET
        FDB     Init           ;COP CLOCK MONITOR FAIL RESET
        FDB     Init           ;RESET

